// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/

#define BAR_LENGTH_1 40
#define BAR_LENGTH_2 5

int currentBar1 = -1, currentBar2a = -1, currentBar2b = 0;
char temps[4] = {'|', '/', '-', '\\'};

// Init progress bar
void progressInit(JNICaller* jniCaller) {
#ifdef UPDATE_PROGRESS
	UPDATE_PROGRESS(*jniCaller, 10.0);
#endif

#ifndef UPDATE_PROGRESS
	string title = " Electrolysis simulation ";
	int total = BAR_LENGTH_1 + 2, amount = round((total - title.length())/2);
	cout << string(amount, '-') << title << string((total - amount - title.length()), '-') << endl;

	cout << ".";
	for (int i = 0; i < BAR_LENGTH_1; i++)
		cout << " ";
	cout << "." << endl << flush;

	currentBar1 = 0;
	currentBar2a = -1;
	currentBar2b = 0;
#endif
}

// Refresh progress bar
void progressPost(int bar1, int bar2a, int bar2b, JNICaller* jniCaller) {
#ifdef UPDATE_PROGRESS
	if (jniCaller != nullptr) {
		float percent = round(90*t/tf) + 10.0;
		if (percent != *jniCaller.percent) {
			UPDATE_PROGRESS(*jniCaller, percent);
			*jniCaller.percent = percent;
		}
	}
#endif
#ifndef UPDATE_PROGRESS
	// Usage of subprogress bar
	if (useSubProgressBar) {
		// Refresh console only if changes
		if ((currentBar1 != bar1) || (currentBar2a != bar2a) || (currentBar2b != bar2b)) {
			cout << '\r';
			// Bar
			cout << "[";
			for (int i = 0; i < BAR_LENGTH_1; i++) {
				if (i < bar1) cout << "#";
				else cout << " ";
			}
			cout << "]";

			// SubBar
			if (bar2b != -1) {
				cout << "(";
				for (int i = 0; i < BAR_LENGTH_2; i++) {
					if ((i < (bar2b-1)) || (bar2b == BAR_LENGTH_2)) cout << "-";
					else if (i == (bar2b-1)) cout << temps[bar2a];
					else cout << " ";
				}
				cout << ")";
			} else {
				for (int i = 0; i < (2+BAR_LENGTH_2); i++) {
					cout << " ";
				}
			}
			cout << flush;
			currentBar1 = bar1;
			currentBar2a = bar2a;
			currentBar2b = bar2b;
		}
	} else {
		// Refresh console only if changes
		if (currentBar1 != bar1) {
			// Bar
			if (currentBar1 == 0) cout << "[";
			for (int i = 0; i < (bar1-currentBar1); i++) cout << "#";
			if (bar1 == BAR_LENGTH_1) cout << "]";
			cout << flush;
			currentBar1 = bar1;
		}
	}
#endif
}
void progressRefresh1(double progress1, JNICaller* jniCaller) {
	int bar1 = mathMinMax(round(progress1*BAR_LENGTH_1), 0, BAR_LENGTH_1);
	progressPost(bar1, -1.0, -1.0, jniCaller);
}
void progressRefresh2b(double progress2b) {
	int bar2b = round(progress2b*BAR_LENGTH_2);
	progressPost(currentBar1, currentBar2a, (bar2b == 0) ? 1 : bar2b, nullptr);
}
void progressRefresh2a() {
	int bar2a = (currentBar2a + 1) % 4;
	if (currentBar2b == -1) progressRefresh2b(0.0);
	progressPost(currentBar1, bar2a, currentBar2b, nullptr);
}





// Error
void errorProgress(const std::string &message) {
#ifndef UPDATE_PROGRESS
	cout << endl << flush;
	cerr << "> Simulation stopped, caused by: " << endl << message << endl << flush;
#endif
}
