// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/

struct Point {
	double x;
	double y;
};
struct Vect {
	double x;
	double y;
};
struct Coord {
	int i;
	int j;
};

struct Topology {
	int polygonCount[2];
	int* polygonIndex[2];
	Point* polygonDefinitions;
};
struct Configuration {
	double L[2];
	double J;
	double tf;
};
struct Performances {
	double electrolysed[2], collected[2], contamination[2], lost[2], remaining[2];
	double resistor;
};

struct TimeSteps {
	double dt, dtElec;
	int i, iElec;
};

struct MatrixTension {
	MatrixXd VALUES;
	double min, max;
	double phi_anode, phi_cathode;
	double resistor;
	double tNext;
	bool sucess;
};

struct Site {
	double x, y, T, growth;
	double nx, ny;
	int type;
	double t, tmiss;
	bool occuped;
};


struct Wall {
	Point P1, P2;
	Vect u, n;
	double length;
};

#define wallMaxmimumIndicators 2
struct WallIndicator {
	Point P;
	Vect n;
	double distance;
	Wall* wall;
};

struct Bubble;
#define interactionMaxAmountAbsolute interactionLongMaximumRange*interactionLongMaximumRange
struct BubbleInteractor {
	int interactionIndexs[interactionMaxAmountAbsolute];
	Bubble* bubbles[interactionMaxAmountAbsolute];
	double distancesSqNorm[interactionMaxAmountAbsolute];
	char imagedModes[interactionMaxAmountAbsolute];
	int count, countVirtual;
	WallIndicator walls[wallMaxmimumIndicators];
};
struct Bubble {
	double x, y, r;
	double ux, uy;
	int statLife, statCollision;
	bool alive;
	Site* site;
	BubbleInteractor interactor;
	double quantity[2];
	double tClogg;
};


struct ListBubbles {
	list<Bubble> attached;
	list<Bubble> detached;
	bool flagAttached, flagDetached;
	int countAttached, countDetached;
	double maxRadius, maxVelocity;
	unsigned long tNextNucleation;
};
struct ElectrolysisStatistics {
	double collected[2], lost[2], contamination[2], remaining[2];
	double resistor, overpotential;
	double tension[2], current[2];
};

struct ElectrodeNode {
	int id;
	int nx_anode, ny_anode;
	int nx_cathode, ny_cathode;
	double denomInv;
};
